/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#ifndef _GB2_ANNOTATION_DATA_H_
#define _GB2_ANNOTATION_DATA_H_

#include <core_api/GObject.h>
#include <core_api/LRegion.h>

#include <QtCore/QSharedData>
#include <QtCore/QVector>
#include <QtCore/QStringList>

namespace GB2 {


class GB2_COREAPI_EXPORT Qualifier {
    friend QDataStream& operator>>(QDataStream& dataStream, Qualifier& q);
    friend QDataStream& operator<<(QDataStream& dataStream, const Qualifier& q);
public:
    Qualifier(){}
    Qualifier(const QString& _name, const QString& _val) : name(_name), val(_val) {}

    bool operator== ( const Qualifier & q ) const { return q.name == name && q.val == val; }
    bool operator!= ( const Qualifier & q ) const { return q.name != name ||  q.val != val; }

    const QString& getQualifierName() const {return name;}

    const QString& getQualifierValue() const {return val;}

private:
    QString name;
    QString val;
};

class GB2_COREAPI_EXPORT AnnotationData : public QSharedData {
public:
    bool                    complement;
    TriState                aminoStrand;
    QString                 name;
    QList<LRegion>          location;
    QVector<Qualifier>      qualifiers;
    
    AnnotationData() : complement(false), aminoStrand(TriState_Unknown) {}

    inline AnnotationData& operator= ( const AnnotationData & a );

    inline void removeAllQualifiers(const QString& name, QStringList& values);

    inline void findQualifiers(const QString& name, QVector<Qualifier>& res) const;

    inline QString findFirstQualifierValue(const QString& name) const;
};

typedef QSharedDataPointer<AnnotationData> SharedAnnotationData;

QDataStream& operator>>(QDataStream& dataStream, TriState& state);
QDataStream& operator<<(QDataStream& dataStream, const TriState& state);
QDataStream& operator>>(QDataStream& dataStream, Qualifier& q);
QDataStream& operator<<(QDataStream& dataStream, const Qualifier& q);
QDataStream& operator>>(QDataStream& dataStream, AnnotationData& data);
QDataStream& operator<<(QDataStream& dataStream, const AnnotationData& data);

AnnotationData& AnnotationData::operator= ( const AnnotationData & a ) { 
    name = a.name; 
    complement = a.complement;
    aminoStrand = a.aminoStrand;
    location = a.location; 
    qualifiers = a.qualifiers;
    return *this;
}

void AnnotationData::removeAllQualifiers(const QString& name, QStringList& values) {
    for (int i=qualifiers.size(); --i>=0;) {
        const Qualifier& q = qualifiers.at(i);
        if (q.getQualifierName() == name) {
            values.append(q.getQualifierValue());
            qualifiers.remove(i);
        }
    }
}

void AnnotationData::findQualifiers(const QString& name, QVector<Qualifier>& res) const {
    foreach (const Qualifier& q, qualifiers) {
        if (q.getQualifierName() == name) {
            res.append(q);
        }
    }
}

QString AnnotationData::findFirstQualifierValue(const QString& name) const {
    foreach (const Qualifier& q, qualifiers) {
        if (q.getQualifierName() == name) {
            return q.getQualifierValue();
        }
    }
    return QString::null;
}

}//namespace

Q_DECLARE_METATYPE(GB2::AnnotationData);
Q_DECLARE_METATYPE(GB2::SharedAnnotationData);
Q_DECLARE_METATYPE(QList<GB2::SharedAnnotationData>);

#endif
