/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#ifndef _GB2_LOGVIEW_IMPL_
#define _GB2_LOGVIEW_IMPL_

#include <core_services/LogSettings.h>

#include <QtCore/QHash>
#include <QtCore/QRegExp>
#include <QtGui/QPlainTextEdit>
#include <QtGui/QLineEdit>
#include <QtGui/QShortcut>
#include <QtGui/QSyntaxHighlighter>

namespace GB2 {

class LogCache;
class LogMessage;

class SearchHighlighter: public QSyntaxHighlighter {
public:
    SearchHighlighter(QTextDocument *doc): QSyntaxHighlighter(doc) {}

    QRegExp reg_exp;
    void highlightBlock(const QString &text);
};

class LogViewDockWidget : public QWidget {
    Q_OBJECT
public:
    LogViewDockWidget(LogCache* c);

    void resetView();
    
    bool isShown(const LogMessage& msg);
    bool isShown(const QString& txt);

    const LogSettings& getSettings() const {return settings;}

    void setSettings(const LogSettings& s); 

protected:
    void addMessage(const LogMessage& msg);
    void addText(const QString& text);
    void showEvent(QShowEvent *e);
    void hideEvent(QHideEvent *e);

private slots:

    void sl_onMessage(const LogMessage& msg);
    void sl_onTextEdited(const QString& text);
    void popupMenu(const QPoint &pos);
    void sl_openSettingsDialog();
    void sl_logSettingsChanged();
    void sl_dumpCounters();
    void sl_clear();
    void sl_addSeparator();
    void sl_showHideEdit();
    void searchPopupMenu(const QPoint &pos);
    void setSearchCaseSensitive();
    void useRegExp();

private:
    struct EntryStruct {
        bool is_plain_text;
        LogMessage msg;
        EntryStruct(const QString &txt): is_plain_text(true) { msg.text = txt; }
        EntryStruct(const LogMessage &_msg): is_plain_text(false), msg(_msg) {}
    };

    QString prepareText(const LogMessage& msg) const;

    QPlainTextEdit*          edit;
    QLineEdit*               search_edit;
    QList<EntryStruct>       original_text;
    QShortcut*               shortcut;
    SearchHighlighter*       highlighter;
    bool case_sensitive, use_regexp;
    void resetText();

	int                 messageCounter;
    LogCache*           cache;
    QAction*            showViewAction;
    QAction*            showSettingsAction;
    QAction*            dumpCountersAction;
    QAction*            clearAction;
    QAction*            addSeparatorAction;

    LogSettings settings;

    bool connected; //for debug only
};

} //namespace

#endif
